@@ -147,11 +147,6 @@ module Sequel
       def database_error_classes
         [::Amalgalite::Error, ::Amalgalite::SQLite3::Error]
       end
-
-      # Disconnect given connections from the database.
-      def disconnect_connection(c)
-        c.close
-      end
     end
     
     # Dataset class for SQLite datasets that use the amalgalite driver.
@@ -176,8 +171,8 @@ module Sequel
       private
       
       # Quote the string using the adapter instance method.
-      def literal_string(v)
-        db.synchronize{|c| c.quote(v)}
+      def literal_string_append(sql, v)
+        db.synchronize{|c| sql << c.quote(v)}
       end
     end
   end
