@@ -1,45 +1,36 @@
 require 'informix'
+Sequel.require 'adapters/shared/informix'
 
 module Sequel
   module Informix
     class Database < Sequel::Database
+      include DatabaseMethods
+
       set_adapter_scheme :informix
       
-      TEMPORARY = 'TEMP '.freeze
-      
       def connect(server)
         opts = server_opts(server)
         ::Informix.connect(opts[:database], opts[:user], opts[:password])
       end
     
-      def dataset(opts = nil)
-        Sequel::Informix::Dataset.new(self, opts)
-      end
-      
       # Returns number of rows affected
       def execute_dui(sql, opts={})
-        log_info(sql)
-        synchronize(opts[:server]){|c| c.immediate(sql)}
+        synchronize(opts[:server]){|c| log_yield(sql){c.immediate(sql)}}
       end
       alias_method :do, :execute_dui
       
       def execute(sql, opts={})
-        log_info(sql)
-        synchronize(opts[:server]){|c| yield c.cursor(sql)}
+        synchronize(opts[:server]){|c| yield log_yield(sql){c.cursor(sql)}}
       end
       alias_method :query, :execute
-      
-      private
-
-      def disconnect_connection(c)
-        c.close
-      end
     end
     
     class Dataset < Sequel::Dataset
-      SELECT_CLAUSE_METHODS = clause_methods(:select, %w'limit distinct columns from join where having group compounds order')
+      include DatasetMethods
 
-      def fetch_rows(sql, &block)
+      Database::DatasetClass = self
+
+      def fetch_rows(sql)
         execute(sql) do |cursor|
           begin
             col_map = nil
@@ -53,27 +44,11 @@ module Sequel
               yield h2
             end
           ensure
-            cursor.drop
+            cursor.respond_to?(:free) ? cursor.free : cursor.drop
           end
         end
         self
       end
-
-      private
-
-      # Informix does not support INTERSECT or EXCEPT
-      def supports_intersect_except?
-        false
-      end
-
-      def select_clause_methods
-        SELECT_CLAUSE_METHODS
-      end
-
-      def select_limit_sql(sql)
-        sql << " SKIP #{@opts[:offset]}" if @opts[:offset]
-        sql << " FIRST #{@opts[:limit]}" if @opts[:limit]
-      end
     end
   end
 end
