@@ -7,24 +7,17 @@ module Sequel
     module MSSQL
       module DatabaseMethods
         include Sequel::MSSQL::DatabaseMethods
-        LAST_INSERT_ID_SQL='SELECT SCOPE_IDENTITY()'
-        
-        # Return an instance of Sequel::ODBC::MSSQL::Dataset with the given opts.
-        def dataset(opts=nil)
-          Sequel::ODBC::MSSQL::Dataset.new(self, opts)
-        end
+        LAST_INSERT_ID_SQL='SELECT SCOPE_IDENTITY()'.freeze
         
         # Return the last inserted identity value.
         def execute_insert(sql, opts={})
-          log_info(sql)
           synchronize(opts[:server]) do |conn|
             begin
-              conn.do(sql)
-              log_info(LAST_INSERT_ID_SQL)
+              log_yield(sql){conn.do(sql)}
               begin
-                s = conn.run(LAST_INSERT_ID_SQL)
-                if (rows = s.fetch_all) and (row = rows.first)
-                  Integer(row.first)
+                s = log_yield(LAST_INSERT_ID_SQL){conn.run(LAST_INSERT_ID_SQL)}
+                if (rows = s.fetch_all) and (row = rows.first) and (v = row.first)
+                  Integer(v)
                 end
               ensure
                 s.drop if s
@@ -35,9 +28,22 @@ module Sequel
           end
         end
       end
-      
       class Dataset < ODBC::Dataset
         include Sequel::MSSQL::DatasetMethods
+
+        private
+
+        # Use ODBC format, not Microsoft format, as the ODBC layer does
+        # some translation.
+        def default_timestamp_format
+          TIMESTAMP_FORMAT
+        end
+
+        # Use ODBC format, not Microsoft format, as the ODBC layer does
+        # some translation.
+        def literal_date(v)
+          v.strftime(ODBC_DATE_FORMAT)
+        end
       end
     end
   end
