@@ -1,4 +1,4 @@
-Sequel.require 'adapters/utils/emulate_offset_with_row_number'
+Sequel.require %w'emulate_offset_with_row_number split_alter_table', 'adapters/utils/'
 
 module Sequel
   Dataset::NON_SQL_OPTIONS << :disable_insert_output
@@ -12,6 +12,10 @@ module Sequel
       SQL_ROLLBACK = "IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION".freeze
       SQL_ROLLBACK_TO_SAVEPOINT = 'IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION autopoint_%d'.freeze
       SQL_SAVEPOINT = 'SAVE TRANSACTION autopoint_%d'.freeze
+      MSSQL_DEFAULT_RE = /\A(?:\(N?('.*')\)|\(\((-?\d+(?:\.\d+)?)\)\))\z/
+      FOREIGN_KEY_ACTION_MAP = {0 => :no_action, 1 => :cascade, 2 => :set_null, 3 => :set_default}.freeze
+
+      include Sequel::Database::SplitAlterTable
       
       # Whether to use N'' to quote strings, which allows unicode characters inside the
       # strings.  True by default for compatibility, can be set to false for a possible
@@ -33,6 +37,47 @@ module Sequel
         false
       end
 
+      # Return foreign key information using the system views, including
+      # :name, :on_delete, and :on_update entries in the hashes.
+      def foreign_key_list(table, opts={})
+        m = output_identifier_meth
+        im = input_identifier_meth
+        schema, table = schema_and_table(table)
+        current_schema = m.call(get(Sequel.function('schema_name')))
+        fk_action_map = FOREIGN_KEY_ACTION_MAP
+        ds = metadata_dataset.from(:sys__foreign_keys___fk).
+          join(:sys__foreign_key_columns___fkc, :constraint_object_id => :object_id).
+          join(:sys__all_columns___pc, :object_id => :fkc__parent_object_id,     :column_id => :fkc__parent_column_id).
+          join(:sys__all_columns___rc, :object_id => :fkc__referenced_object_id, :column_id => :fkc__referenced_column_id).
+          where{{object_schema_name(:fk__parent_object_id) => im.call(schema || current_schema)}}.
+          where{{object_name(:fk__parent_object_id) => im.call(table)}}.
+          select{[:fk__name, 
+                  :fk__delete_referential_action, 
+                  :fk__update_referential_action, 
+                  :pc__name___column, 
+                  :rc__name___referenced_column, 
+                  object_schema_name(:fk__referenced_object_id).as(:schema), 
+                  object_name(:fk__referenced_object_id).as(:table)]}.
+          order(:name, :fkc__constraint_column_id)
+        h = {}
+        ds.each do |row|
+          if r = h[row[:name]]
+            r[:columns] << m.call(row[:column])
+            r[:key] << m.call(row[:referenced_column])
+          else
+            referenced_schema = m.call(row[:schema])
+            referenced_table = m.call(row[:table])
+            h[row[:name]] = { :name      => m.call(row[:name]), 
+                              :table     => (referenced_schema == current_schema) ? referenced_table : :"#{referenced_schema}__#{referenced_table}",
+                              :columns   => [m.call(row[:column])], 
+                              :key       => [m.call(row[:referenced_column])], 
+                              :on_update => fk_action_map[row[:update_referential_action]], 
+                              :on_delete => fk_action_map[row[:delete_referential_action]] }
+          end
+        end
+        h.values
+      end
+
       # Use the system tables to get index information
       def indexes(table, opts={})
         m = output_identifier_meth
@@ -93,9 +138,17 @@ module Sequel
       def views(opts={})
         information_schema_tables('VIEW', opts)
       end
-
+      
       private
       
+      # Add dropping of the default constraint to the list of SQL queries.
+      # This is necessary before dropping the column or changing its type.
+      def add_drop_default_constraint_sql(sqls, table, column)
+        if constraint = default_constraint_name(table, column)
+          sqls << "ALTER TABLE #{quote_schema_table(table)} DROP CONSTRAINT #{constraint}"
+        end
+      end
+
       # MSSQL uses the IDENTITY(1,1) column for autoincrementing columns.
       def auto_increment_sql
         AUTO_INCREMENT
@@ -106,6 +159,10 @@ module Sequel
         case op[:op]
         when :add_column
           "ALTER TABLE #{quote_schema_table(table)} ADD #{column_definition_sql(op)}"
+        when :drop_column
+          sqls = []
+          add_drop_default_constraint_sql(sqls, table, op[:name])
+          sqls << super
         when :rename_column
           "sp_rename #{literal("#{quote_schema_table(table)}.#{quote_identifier(op[:name])}")}, #{literal(op[:new_name].to_s)}, 'COLUMN'"
         when :set_column_type
@@ -113,9 +170,7 @@ module Sequel
           if sch = schema(table)
             if cs = sch.each{|k, v| break v if k == op[:name]; nil}
               cs = cs.dup
-              if constraint = default_constraint_name(table, op[:name])
-                sqls << "ALTER TABLE #{quote_schema_table(table)} DROP CONSTRAINT #{constraint}"
-              end
+              add_drop_default_constraint_sql(sqls, table, op[:name])
               cs[:default] = cs[:ruby_default]
               op = cs.merge!(op)
               default = op.delete(:default)
@@ -148,6 +203,14 @@ module Sequel
         SQL_BEGIN
       end
       
+      # Handle MSSQL specific default format.
+      def column_schema_normalize_default(default, type)
+        if m = MSSQL_DEFAULT_RE.match(default)
+          default = m[1] || m[2]
+        end
+        super(default, type)
+      end
+
       # Commit the active transaction on the connection, does not commit/release
       # savepoints.
       def commit_transaction(conn, opts={})
@@ -176,15 +239,15 @@ module Sequel
       end
     
       # The name of the constraint for setting the default value on the table and column.
-      def default_constraint_name(table, column)
-        from(:sysobjects___c_obj).
-          join(:syscomments___com, :id=>:id).
-          join(:sysobjects___t_obj, :id=>:c_obj__parent_obj).
-          join(:sysconstraints___con, :constid=>:c_obj__id).
-          join(:syscolumns___col, :id=>:t_obj__id, :colid=>:colid).
-          where{{c_obj__uid=>user_id{}}}.
-          where(:c_obj__xtype=>'D', :t_obj__name=>table.to_s, :col__name=>column.to_s).
-          get(:c_obj__name)
+      # The SQL used to select default constraints utilizes MSSQL catalog views which were introduced in 2005.
+      # This method intentionally does not support MSSQL 2000.
+      def default_constraint_name(table, column_name)
+        if server_version >= 9000000
+          table_name = schema_and_table(table).compact.join('.')
+          self[:sys__default_constraints].
+            where{{:parent_object_id => Sequel::SQL::Function.new(:object_id, table_name), col_name(:parent_object_id, :parent_column_id) => column_name.to_s}}.
+            get(:name)
+        end
       end
 
       # The SQL to drop an index for the table.
@@ -251,15 +314,19 @@ module Sequel
         m2 = input_identifier_meth(opts[:dataset])
         tn = m2.call(table_name.to_s)
         table_id = get{object_id(tn)}
-        pk_index_id = metadata_dataset.from(:sysindexes).
+        info_sch_sch = opts[:information_schema_schema]
+        inf_sch_qual = lambda{|s| info_sch_sch ? Sequel.qualify(info_sch_sch, s) : Sequel.expr(s)}
+        sys_qual = lambda{|s| info_sch_sch ? Sequel.qualify(info_sch_sch, Sequel.qualify(Sequel.lit(''), s)) : Sequel.expr(s)}
+
+        pk_index_id = metadata_dataset.from(sys_qual.call(:sysindexes)).
           where(:id=>table_id, :indid=>1..254){{(status & 2048)=>2048}}.
           get(:indid)
-        pk_cols = metadata_dataset.from(:sysindexkeys___sik).
-          join(:syscolumns___sc, :id=>:id, :colid=>:colid).
+        pk_cols = metadata_dataset.from(sys_qual.call(:sysindexkeys).as(:sik)).
+          join(sys_qual.call(:syscolumns).as(:sc), :id=>:id, :colid=>:colid).
           where(:sik__id=>table_id, :sik__indid=>pk_index_id).
           select_order_map(:sc__name)
-        ds = metadata_dataset.from(:information_schema__tables___t).
-         join(:information_schema__columns___c, :table_catalog=>:table_catalog,
+        ds = metadata_dataset.from(inf_sch_qual.call(:information_schema__tables).as(:t)).
+         join(inf_sch_qual.call(:information_schema__columns).as(:c), :table_catalog=>:table_catalog,
               :table_schema => :table_schema, :table_name => :table_name).
          select(:column_name___column, :data_type___db_type, :character_maximum_length___max_chars, :column_default___default, :is_nullable___allow_null, :numeric_precision___column_size, :numeric_scale___scale).
          filter(:c__table_name=>tn)
@@ -317,6 +384,7 @@ module Sequel
       INSERT_CLAUSE_METHODS = Dataset.clause_methods(:insert, %w'with insert into columns output values')
       SELECT_CLAUSE_METHODS = Dataset.clause_methods(:select, %w'with select distinct limit columns into from lock join where group having order compounds')
       UPDATE_CLAUSE_METHODS = Dataset.clause_methods(:update, %w'with update limit table set output from where')
+      UPDATE_CLAUSE_METHODS_2000 = Dataset.clause_methods(:update, %w'update table set output from where')
       NOLOCK = ' WITH (NOLOCK)'.freeze
       UPDLOCK = ' WITH (UPDLOCK)'.freeze
       WILDCARD = LiteralString.new('*').freeze
@@ -411,6 +479,21 @@ module Sequel
         mutation_method(:disable_insert_output)
       end
 
+      # There is no function on Microsoft SQL Server that does character length
+      # and respects trailing spaces (datalength respects trailing spaces, but
+      # counts bytes instead of characters).  Use a hack to work around the
+      # trailing spaces issue.
+      def emulated_function_sql_append(sql, f)
+        case f.f
+        when :char_length
+          literal_append(sql, SQL::Function.new(:len, Sequel.join([f.args.first, 'x'])) - 1)
+        when :trim
+          literal_append(sql, SQL::Function.new(:ltrim, SQL::Function.new(:rtrim, f.args.first)))
+        else
+          super
+        end
+      end
+      
       # MSSQL uses the CONTAINS keyword for full text search
       def full_text_search(cols, terms, opts = {})
         terms = "\"#{terms.join('" OR "')}\"" if terms.is_a?(Array)
@@ -723,10 +806,15 @@ module Sequel
       alias delete_output_sql output_sql
       alias update_output_sql output_sql
 
-      # MSSQL supports the OUTPUT clause for UPDATE statements.
-      # It also allows prepending a WITH clause.
+      # MSSQL supports the OUTPUT and TOP clause for UPDATE statements.
+      # It also allows prepending a WITH clause.  For MSSQL 2000
+      # and below, exclude WITH and TOP.
       def update_clause_methods
-        UPDATE_CLAUSE_METHODS
+        if is_2005_or_later?
+          UPDATE_CLAUSE_METHODS
+        else
+          UPDATE_CLAUSE_METHODS_2000
+        end
       end
 
       # Only include the primary table in the main update clause
