@@ -9,7 +9,7 @@ module Sequel
     end
   
     module DatasetMethods
-      SELECT_CLAUSE_METHODS = Dataset.clause_methods(:select, %w'limit distinct columns from join where group order having compounds')
+      SELECT_CLAUSE_METHODS = Dataset.clause_methods(:select, %w'select limit distinct columns from join where group order having compounds')
 
       # Progress requires SQL standard datetimes
       def requires_sql_standard_datetimes?
@@ -31,7 +31,10 @@ module Sequel
       # The Progress adapter targets Progress 9, so it silently ignores the option.
       def select_limit_sql(sql)
         raise(Error, "OFFSET not supported") if @opts[:offset]
-        #sql << " TOP #{@opts[:limit]}" if @opts[:limit]
+        # if l = @opts[:limit]
+        #   sql << " TOP "
+        #   literal_append(sql, l)
+        # end
       end
     end
   end
