@@ -58,11 +58,14 @@ module Sequel
       a
     end
     
-    # Returns the average value for the given column.
+    # Returns the average value for the given column/expression.
+    # Uses a virtual row block if no argument is given.
     #
     #   DB[:table].avg(:number) # SELECT avg(number) FROM table LIMIT 1
     #   # => 3
-    def avg(column)
+    #   DB[:table].avg{function(column)} # SELECT avg(function(column)) FROM table LIMIT 1
+    #   # => 1
+    def avg(column=Sequel.virtual_row(&Proc.new))
       aggregate_dataset.get{avg(column)}
     end
   
@@ -93,12 +96,30 @@ module Sequel
       columns
     end
     
-    # Returns the number of records in the dataset.
+    # Returns the number of records in the dataset. If an argument is provided,
+    # it is used as the argument to count.  If a block is provided, it is
+    # treated as a virtual row, and the result is used as the argument to
+    # count.
     #
     #   DB[:table].count # SELECT COUNT(*) AS count FROM table LIMIT 1
     #   # => 3
-    def count
-      aggregate_dataset.get{COUNT(:*){}.as(count)}.to_i
+    #   DB[:table].count(:column) # SELECT COUNT(column) AS count FROM table LIMIT 1
+    #   # => 2
+    #   DB[:table].count{foo(column)} # SELECT COUNT(foo(column)) AS count FROM table LIMIT 1
+    #   # => 1
+    def count(arg=(no_arg=true), &block)
+      if no_arg
+        if block
+          arg = Sequel.virtual_row(&block)
+          aggregate_dataset.get{COUNT(arg).as(count)}
+        else
+          aggregate_dataset.get{COUNT(:*){}.as(count)}.to_i
+        end
+      elsif block
+        raise Error, 'cannot provide both argument and block to Dataset#count'
+      else
+        aggregate_dataset.get{COUNT(arg).as(count)}
+      end
     end
     
     # Deletes the records in the dataset.  The returned value should be 
@@ -204,12 +225,35 @@ module Sequel
     #
     #   ds.get{sum(id)} # SELECT sum(id) FROM table LIMIT 1
     #   # => 6
+    #
+    # You can pass an array of arguments to return multiple arguments,
+    # but you must make sure each element in the array has an alias that
+    # Sequel can determine:
+    #
+    #   DB[:table].get([:id, :name]) # SELECT id, name FROM table LIMIT 1
+    #   # => [3, 'foo']
+    #
+    #   DB[:table].get{[sum(id).as(sum), name]} # SELECT sum(id) AS sum, name FROM table LIMIT 1
+    #   # => [6, 'foo']
     def get(column=(no_arg=true; nil), &block)
+      ds = naked
       if block
         raise(Error, ARG_BLOCK_ERROR_MSG) unless no_arg
-        select(&block).single_value
+        ds = ds.select(&block)
+        column = ds.opts[:select]
+        column = nil if column.is_a?(Array) && column.length < 2
       else
-        select(column).single_value
+        ds = if column.is_a?(Array)
+          ds.select(*column)
+        else
+          ds.select(column)
+        end
+      end
+
+      if column.is_a?(Array)
+        ds.single_record.values_at(*column.map{|c| hash_key_symbol(c)})
+      else
+        ds.single_value
       end
     end
     
@@ -311,7 +355,7 @@ module Sequel
     #   # INSERT INTO table (x) VALUES (1)
     #   # INSERT INTO table (x) VALUES (2)
     #
-    #   DB[:table].insert_multiple([{:x=>1}, {:x=>2}]){|row| row[:y] = row[:x] * 2}
+    #   DB[:table].insert_multiple([{:x=>1}, {:x=>2}]){|row| row[:y] = row[:x] * 2; row }
     #   # => [6, 7]
     #   # INSERT INTO table (x, y) VALUES (1, 2)
     #   # INSERT INTO table (x, y) VALUES (2, 4)
@@ -324,11 +368,13 @@ module Sequel
     end
     
     # Returns the interval between minimum and maximum values for the given 
-    # column.
+    # column/expression. Uses a virtual row block if no argument is given.
     #
     #   DB[:table].interval(:id) # SELECT (max(id) - min(id)) FROM table LIMIT 1
     #   # => 6
-    def interval(column)
+    #   DB[:table].interval{function(column)} # SELECT (max(function(column)) - min(function(column))) FROM table LIMIT 1
+    #   # => 7
+    def interval(column=Sequel.virtual_row(&Proc.new))
       aggregate_dataset.get{max(column) - min(column)}
     end
 
@@ -340,7 +386,7 @@ module Sequel
     #   DB[:table].order(:id).last # SELECT * FROM table ORDER BY id DESC LIMIT 1
     #   # => {:id=>10}
     #
-    #   DB[:table].order(:id.desc).last(2) # SELECT * FROM table ORDER BY id ASC LIMIT 2
+    #   DB[:table].order(Sequel.desc(:id)).last(2) # SELECT * FROM table ORDER BY id ASC LIMIT 2
     #   # => [{:id=>1}, {:id=>2}]
     def last(*args, &block)
       raise(Error, 'No order specified') unless @opts[:order]
@@ -375,19 +421,25 @@ module Sequel
       end
     end
 
-    # Returns the maximum value for the given column.
+    # Returns the maximum value for the given column/expression.
+    # Uses a virtual row block if no argument is given.
     #
     #   DB[:table].max(:id) # SELECT max(id) FROM table LIMIT 1
     #   # => 10
-    def max(column)
+    #   DB[:table].max{function(column)} # SELECT max(function(column)) FROM table LIMIT 1
+    #   # => 7
+    def max(column=Sequel.virtual_row(&Proc.new))
       aggregate_dataset.get{max(column)}
     end
 
-    # Returns the minimum value for the given column.
+    # Returns the minimum value for the given column/expression.
+    # Uses a virtual row block if no argument is given.
     #
     #   DB[:table].min(:id) # SELECT min(id) FROM table LIMIT 1
     #   # => 1
-    def min(column)
+    #   DB[:table].min{function(column)} # SELECT min(function(column)) FROM table LIMIT 1
+    #   # => 0
+    def min(column=Sequel.virtual_row(&Proc.new))
       aggregate_dataset.get{min(column)}
     end
 
@@ -410,11 +462,13 @@ module Sequel
     end
 
     # Returns a +Range+ instance made from the minimum and maximum values for the
-    # given column.
+    # given column/expression.  Uses a virtual row block if no argument is given.
     #
     #   DB[:table].range(:id) # SELECT max(id) AS v1, min(id) AS v2 FROM table LIMIT 1
     #   # => 1..10
-    def range(column)
+    #   DB[:table].interval{function(column)} # SELECT max(function(column)) AS v1, min(function(column)) AS v2 FROM table LIMIT 1
+    #   # => 0..7
+    def range(column=Sequel.virtual_row(&Proc.new))
       if r = aggregate_dataset.select{[min(column).as(v1), max(column).as(v2)]}.first
         (r[:v1]..r[:v2])
       end
@@ -525,11 +579,14 @@ module Sequel
       end
     end
     
-    # Returns the sum for the given column.
+    # Returns the sum for the given column/expression.
+    # Uses a virtual row block if no column is given.
     #
     #   DB[:table].sum(:id) # SELECT sum(id) FROM table LIMIT 1
     #   # => 55
-    def sum(column)
+    #   DB[:table].sum{function(column)} # SELECT sum(function(column)) FROM table LIMIT 1
+    #   # => 10
+    def sum(column=Sequel.virtual_row(&Proc.new))
       aggregate_dataset.get{sum(column)}
     end
 
@@ -768,7 +825,7 @@ module Sequel
       when Symbol
         _, c, a = split_symbol(s)
         (a || c).to_sym
-      when SQL::Identifier
+      when SQL::Identifier, SQL::Wrapper
         hash_key_symbol(s.value)
       when SQL::QualifiedIdentifier
         hash_key_symbol(s.column)
@@ -820,10 +877,9 @@ module Sequel
       when SQL::AliasedExpression
         c.expression
       when SQL::OrderedExpression
-        expr = c.expression
-        if expr.is_a?(Symbol)
-          expr = unaliased_identifier(expr)
-          SQL::OrderedExpression.new(unaliased_identifier(c.expression), c.descending, :nulls=>c.nulls)
+        case expr = c.expression
+        when Symbol, SQL::AliasedExpression
+          SQL::OrderedExpression.new(unaliased_identifier(expr), c.descending, :nulls=>c.nulls)
         else
           c
         end
