@@ -8,6 +8,8 @@
 #
 #   Sequel.extension :schema_dumper
 
+Sequel.extension :eval_inspect
+
 module Sequel
   class Database
     # Dump foreign key constraints for all tables as a migration. This complements
@@ -97,14 +99,13 @@ END_MIG
 
     private
         
-    # If a database default exists and can't be converted, return the string with the inspect
-    # method modified so that .lit is always appended after it, only if the
-    # :same_db option is used.
+    # If a database default exists and can't be converted, and we are dumping with :same_db,
+    # return a string with the inspect method modified a literal string is created if the code is evaled.  
     def column_schema_to_ruby_default_fallback(default, options)
       if default.is_a?(String) && options[:same_db] && use_column_schema_to_ruby_default_fallback?
-        default = default.to_s
+        default = default.dup
         def default.inspect
-          "#{super}.lit"  # core_sql use
+          "Sequel::LiteralString.new(#{super})"
         end
         default
       end
@@ -137,8 +138,7 @@ END_MIG
           gen.foreign_key(name, table, col_opts)
         else
           gen.column(name, type, col_opts)
-          if (type == Integer || type == Bignum) && schema[:db_type] =~ / unsigned\z/io
-            Sequel.extension :eval_inspect
+          if [Integer, Bignum, Float].include?(type) && schema[:db_type] =~ / unsigned\z/io
             gen.check(Sequel::SQL::Identifier.new(name) >= 0)
           end
         end
@@ -162,7 +162,7 @@ END_MIG
         {:type =>schema[:type] == :boolean ? TrueClass : Integer}
       when /\Abigint(?:\((?:\d+)\))?(?: unsigned)?\z/o
         {:type=>Bignum}
-      when /\A(?:real|float|double(?: precision)?)\z/o
+      when /\A(?:real|float|double(?: precision)?|double\(\d+,\d+\)(?: unsigned)?)\z/o
         {:type=>Float}
       when 'boolean'
         {:type=>TrueClass}
@@ -304,6 +304,7 @@ END_MIG
         end
       end
       h[:unique] = true if index_opts[:unique]
+      h[:deferrable] = true if index_opts[:deferrable]
       h
     end
 
@@ -371,7 +372,7 @@ END_MIG
       [sorted_tables, skipped_foreign_keys]
     end
     
-    # Don't use the "...".lit fallback on MySQL, since the defaults it uses aren't
+    # Don't use a literal string fallback on MySQL, since the defaults it uses aren't
     # valid literal SQL values.
     def use_column_schema_to_ruby_default_fallback?
       database_type != :mysql
@@ -474,16 +475,7 @@ END_MIG
       def opts_inspect(opts)
         if opts[:default]
           opts = opts.dup
-          de = case d = opts.delete(:default)
-          when BigDecimal, Sequel::SQL::Blob
-            "#{d.class.name}.new(#{d.to_s.inspect})"
-          when DateTime, Date
-            "#{d.class.name}.parse(#{d.to_s.inspect})"
-          when Time
-            "#{d.class.name}.parse(#{d.strftime('%H:%M:%S').inspect})"
-          else
-            d.inspect
-          end
+          de = Sequel.eval_inspect(opts.delete(:default)) 
           ", :default=>#{de}#{", #{opts.inspect[1...-1]}" if opts.length > 0}"
         else
           ", #{opts.inspect[1...-1]}" if opts.length > 0
