@@ -2,8 +2,7 @@ require 'rubygems'
 require 'logger'
 unless Object.const_defined?('Sequel')
   $:.unshift(File.join(File.dirname(File.expand_path(__FILE__)), "../../lib/"))
-  require 'sequel'
-  Sequel.quote_identifiers = false
+  require 'sequel/no_core_ext'
 end
 begin
   require File.join(File.dirname(File.dirname(File.expand_path(__FILE__))), 'spec_config.rb')
@@ -15,6 +14,8 @@ if ENV['SEQUEL_COLUMNS_INTROSPECTION']
   Sequel::Dataset.introspect_all_columns
 end
 
+Sequel::Model.cache_anonymous_models = false
+
 class Sequel::Database
   def log_duration(duration, message)
     log_info(message)
@@ -31,11 +32,8 @@ end
     end 
   end 
 
-  def self.log_specify(message, &block)
-    specify(message){log{instance_eval(&block)}}
-  end
-
   def self.cspecify(message, *checked, &block)
+    return specify(message, &block) if ENV['SEQUEL_NO_PENDING']
     pending = false
     checked.each do |c|
       case c
@@ -53,4 +51,11 @@ end
       specify(message, &block)
     end
   end
+
+  def check_sqls
+    yield unless ENV['SEQUEL_NO_CHECK_SQLS']
+  end
+  def self.check_sqls
+    yield unless ENV['SEQUEL_NO_CHECK_SQLS']
+  end
 end
