@@ -1,8 +1,9 @@
 require File.join(File.dirname(File.expand_path(__FILE__)), "spec_helper")
+require 'yaml'
 
 describe "Sequel::Plugins::LazyAttributes" do
   before do
-    @db = MockDatabase.new
+    @db = Sequel.mock
     @db.meta_def(:schema){|*a| [[:id, {:type=>:integer}], [:name,{:type=>:string}]]}
     class ::LazyAttributesModel < Sequel::Model(@db[:la])
       plugin :lazy_attributes
@@ -10,27 +11,23 @@ describe "Sequel::Plugins::LazyAttributes" do
       meta_def(:columns){[:id, :name]}
       lazy_attributes :name
       meta_def(:columns){[:id]}
-      ds = dataset
-      def ds.fetch_rows(sql)
-        execute(sql)
-        select = @opts[:select]
-        where = @opts[:where]
-        block = @mod_block || proc{|s| s}
-        if !where
-          if select.include?(:name)
-            yield(block[:id=>1, :name=>'1'])
-            yield(block[:id=>2, :name=>'2'])
+      instance_dataset._fetch = dataset._fetch = proc do |sql|
+        if sql !~ /WHERE/
+          if sql =~ /name/
+            [{:id=>1, :name=>'1'}, {:id=>2, :name=>'2'}]
           else
-            yield(:id=>1)
-            yield(:id=>2)
+            [{:id=>1}, {:id=>2}]
           end
         else
-          i = where.args.last
-          Array(i).each do |x|
+          if sql =~ /id IN \(([\d, ]+)\)/
+            $1.split(', ')
+          elsif sql =~ /id = (\d)/
+            [$1]
+          end.map do |x|
             if sql =~ /SELECT name FROM/
-              yield(block[:name=>x.to_s])
+              {:name=>x.to_s}
             else
-              yield(block[:id=>x, :name=>x.to_s])
+              {:id=>x.to_i, :name=>x.to_s}
             end
           end
         end
@@ -38,7 +35,7 @@ describe "Sequel::Plugins::LazyAttributes" do
     end
     @c = ::LazyAttributesModel
     @ds = LazyAttributesModel.dataset
-    @db.reset
+    @db.sqls
   end
   after do
     Object.send(:remove_const, :LazyAttributesModel)
@@ -136,7 +133,7 @@ describe "Sequel::Plugins::LazyAttributes" do
 
   it "should work with the serialization plugin" do
     @c.plugin :serialization, :yaml, :name
-    @ds.instance_variable_set(:@mod_block, proc{|s| s.merge(:name=>"--- #{s[:name].to_i*3}\n")})
+    @c.instance_dataset._fetch = @ds._fetch = [[{:id=>1}, {:id=>2}], [{:id=>1, :name=>"--- 3\n"}, {:id=>2, :name=>"--- 6\n"}], [{:id=>1}], [{:name=>"--- 3\n"}]]
     @c.with_identity_map do
       ms = @ds.all
       ms.map{|m| m.values}.should == [{:id=>1}, {:id=>2}]
@@ -146,7 +143,6 @@ describe "Sequel::Plugins::LazyAttributes" do
       ms.map{|m| m.name}.should == [3,6]
       @db.sqls.should == ['SELECT id FROM la', 'SELECT id, name FROM la WHERE (id IN (1, 2))']
     end
-    @db.reset
     @c.with_identity_map do
       m = @ds.first
       m.values.should == {:id=>1}
