@@ -1,8 +1,15 @@
 require File.join(File.dirname(File.expand_path(__FILE__)), "spec_helper")
 
+model_class = proc do |klass, &block|
+  c = Class.new(klass)
+  c.plugin :validation_class_methods
+  c.class_eval(&block) if block
+  c
+end
+
 describe Sequel::Model do
   before do
-    @c = Class.new(Sequel::Model) do
+    @c = model_class.call Sequel::Model do
       def self.validates_coolness_of(attr)
         validates_each(attr) {|o, a, v| o.errors[a] << 'is not cool' if v != :cool}
       end
@@ -135,8 +142,7 @@ end
 
 describe Sequel::Model do
   before do
-    @c = Class.new(Sequel::Model)
-    @c.class_eval do
+    @c = model_class.call Sequel::Model do
       columns :score
       validates_each :score do |o, a, v|
         o.errors[a] << 'too low' if v < 87
@@ -165,11 +171,11 @@ describe Sequel::Model do
   end
 end
 
-describe Sequel::Plugins::ValidationClassMethods::ClassMethods::Generator do
+describe "Sequel::Plugins::ValidationClassMethods::ClassMethods::Generator" do
   before do
     $testit = nil
     
-    @c = Class.new(Sequel::Model) do
+    @c = model_class.call Sequel::Model do
       def self.validates_blah
         $testit = 1324
       end
@@ -186,7 +192,7 @@ end
 
 describe Sequel::Model do
   before do
-    @c = Class.new(Sequel::Model) do
+    @c = model_class.call Sequel::Model do
       columns :value
       
       def self.filter(*args)
@@ -527,13 +533,8 @@ describe Sequel::Model do
   end
   
   specify "should raise an error if inclusion_of doesn't receive a valid :in option" do
-    lambda {
-      @c.validates_inclusion_of :value
-    }.should raise_error(ArgumentError)
-    
-    lambda {
-      @c.validates_inclusion_of :value, :in => 1
-    }.should raise_error(ArgumentError)
+    lambda{@c.validates_inclusion_of :value}.should raise_error(ArgumentError)
+    lambda{@c.validates_inclusion_of :value, :in => 1}.should raise_error(ArgumentError)
   end
   
   specify "should raise an error if inclusion_of handles :allow_nil too" do
@@ -570,7 +571,6 @@ describe Sequel::Model do
 
   specify "should validate uniqueness_of with if => false" do
     @c.validates_uniqueness_of :value, :if => :skip
-
     @m.value = 'a'
     @m.should be_valid
   end
@@ -578,15 +578,14 @@ describe Sequel::Model do
   specify "should validate uniqueness_of with allow_missing => true" do
     @c.validates_uniqueness_of :value, :allow_missing => true
     @m.should be_valid
-    @m.value = nil
+    @m.value = 1
     @m.should_not be_valid
   end
 end
 
 describe "Superclass validations" do
   before do
-    @c1 = Class.new(Sequel::Model)
-    @c1.class_eval do
+    @c1 = model_class.call Sequel::Model do
       columns :value
       validates_length_of :value, :minimum => 5
     end
@@ -602,16 +601,11 @@ describe "Superclass validations" do
     o = @c2.new
     o.value = 'ab'
     o.valid?.should == false
-    o.errors.full_messages.should == [
-      'value is too short'
-    ]
+    o.errors.full_messages.should == ['value is too short']
 
     o.value = '12'
     o.valid?.should == false
-    o.errors.full_messages.should == [
-      'value is too short',
-      'value is invalid'
-    ]
+    o.errors.full_messages.should == ['value is too short', 'value is invalid']
 
     o.value = 'abcde'
     o.valid?.should be_true
@@ -626,9 +620,7 @@ describe "Superclass validations" do
 
     o.value = '12'
     o.valid?.should == false
-    o.errors.full_messages.should == [
-      'value is invalid'
-    ]
+    o.errors.full_messages.should == ['value is invalid']
 
     o.value = 'abcde'
     o.valid?.should be_true
@@ -637,8 +629,7 @@ end
 
 describe ".validates with block" do
   specify "should support calling .each" do
-    @c = Class.new(Sequel::Model)
-    @c.class_eval do
+    @c = model_class.call Sequel::Model do
       columns :vvv
       validates do
         each :vvv do |o, a, v|
@@ -659,24 +650,36 @@ describe Sequel::Model, "Validations" do
 
   before(:all) do
     class ::Person < Sequel::Model
+      plugin :validation_class_methods
       columns :id,:name,:first_name,:last_name,:middle_name,:initials,:age, :terms
     end
 
     class ::Smurf < Person
     end
+
+    class ::Can < Sequel::Model
+      plugin :validation_class_methods
+      columns :id, :name
+    end
     
     class ::Cow < Sequel::Model
+      plugin :validation_class_methods
       columns :id, :name, :got_milk
     end
 
     class ::User < Sequel::Model
+      plugin :validation_class_methods
       columns :id, :username, :password
     end
     
     class ::Address < Sequel::Model
+      plugin :validation_class_methods
       columns :id, :zip_code
     end
   end
+  after(:all) do
+    [:Person, :Smurf, :Cow, :User, :Address].each{|c| Object.send(:remove_const, c)}
+  end
   
   it "should validate the acceptance of a column" do
     class ::Cow < Sequel::Model
@@ -754,8 +757,7 @@ describe Sequel::Model, "Validations" do
   end
   
   it "should validate that a column doesn't have a string value" do
-    p = Class.new(Sequel::Model)
-    p.class_eval do
+    p = model_class.call Sequel::Model do
       columns :age, :price, :confirmed
       self.raise_on_typecast_failure = false
       validates_not_string :age
@@ -824,22 +826,18 @@ describe Sequel::Model, "Validations" do
         uniqueness_of :username
       end
     end
-    User.dataset.extend(Module.new {
-      def fetch_rows(sql)
-        @db << sql
-        
-        case sql
-        when /COUNT.*username = '0records'/
-          yield({:v => 0})
-        when /COUNT.*username = '2records'/
-          yield({:v => 2})
-        when /COUNT.*username = '1record'/
-          yield({:v => 1})
-        when /username = '1record'/
-          yield({:id => 3, :username => "1record", :password => "test"})
-        end
+    User.dataset._fetch = proc do |sql|
+      case sql
+      when /COUNT.*username = '0records'/
+        {:v => 0}
+      when /COUNT.*username = '2records'/
+        {:v => 2}
+      when /COUNT.*username = '1record'/
+        {:v => 1}
+      when /username = '1record'/
+        {:id => 3, :username => "1record", :password => "test"}
       end
-    })
+    end
     
     @user = User.new(:username => "2records", :password => "anothertest")
     @user.should_not be_valid
@@ -860,6 +858,12 @@ describe Sequel::Model, "Validations" do
     @user = User.new(:username => "0records", :password => "anothertest")
     @user.should be_valid
     @user.errors.full_messages.should == []
+
+    User.db.sqls
+    @user = User.new(:password => "anothertest")
+    @user.should be_valid
+    @user.errors.full_messages.should == []
+    User.db.sqls.should == []
   end
   
   it "should validate the uniqueness of multiple columns" do
@@ -869,26 +873,22 @@ describe Sequel::Model, "Validations" do
         uniqueness_of [:username, :password]
       end
     end
-    User.dataset.extend(Module.new {
-      def fetch_rows(sql)
-        @db << sql
-        
-        case sql
-        when /COUNT.*username = '0records'/
-          yield({:v => 0})
-        when /COUNT.*username = '2records'/
-          yield({:v => 2})
-        when /COUNT.*username = '1record'/
-          yield({:v => 1})
-        when /username = '1record'/
-          if sql =~ /password = 'anothertest'/
-            yield({:id => 3, :username => "1record", :password => "anothertest"})
-          else
-            yield({:id => 4, :username => "1record", :password => "test"})
-          end
+    User.dataset._fetch = proc do |sql|
+      case sql
+      when /COUNT.*username = '0records'/
+        {:v => 0}
+      when /COUNT.*username = '2records'/
+        {:v => 2}
+      when /COUNT.*username = '1record'/
+        {:v => 1}
+      when /username = '1record'/
+        if sql =~ /password = 'anothertest'/
+          {:id => 3, :username => "1record", :password => "anothertest"}
+        else
+          {:id => 4, :username => "1record", :password => "test"}
         end
       end
-    })
+    end
     
     @user = User.new(:username => "2records", :password => "anothertest")
     @user.should_not be_valid
@@ -913,6 +913,18 @@ describe Sequel::Model, "Validations" do
     @user = User.new(:username => "0records", :password => "anothertest")
     @user.should be_valid
     @user.errors.full_messages.should == []
+
+    User.db.sqls
+    @user = User.new(:password => "anothertest")
+    @user.should be_valid
+    @user.errors.full_messages.should == []
+    @user = User.new(:username => "0records")
+    @user.should be_valid
+    @user.errors.full_messages.should == []
+    @user = User.new
+    @user.should be_valid
+    @user.errors.full_messages.should == []
+    User.db.sqls.should == []
   end
   
   it "should have a validates block that contains multiple validations" do
@@ -962,8 +974,6 @@ describe Sequel::Model, "Validations" do
 
   it "should validate correctly instances initialized with string keys" do
     class ::Can < Sequel::Model
-      columns :id, :name
-      
       validates_length_of :name, :minimum => 4
     end
     
@@ -975,8 +985,7 @@ end
 
 describe "Model#save" do
   before do
-    @c = Class.new(Sequel::Model(:people))
-    @c.class_eval do
+    @c = model_class.call Sequel::Model(:people) do
       columns :id, :x
 
       validates_each :x do |o, a, v|
