@@ -13,18 +13,25 @@ describe "Prepared Statements and Bound Arguments" do
     @pr = @ds.requires_placeholder_type_specifiers? ? proc{|i| :"#{i}__integer"} : proc{|i| i}
   end
   after do
-    @db.drop_table(:items)
+    @db.drop_table?(:items)
   end
   
-  specify "should support bound variables with select, all, and first" do
+  specify "should support bound variables when selecting" do
+    @ds.filter(:numb=>:$n).call(:each, :n=>10){|h| h.should == {:id=>1, :numb=>10}}
     @ds.filter(:numb=>:$n).call(:select, :n=>10).should == [{:id=>1, :numb=>10}]
     @ds.filter(:numb=>:$n).call(:all, :n=>10).should == [{:id=>1, :numb=>10}]
     @ds.filter(:numb=>:$n).call(:first, :n=>10).should == {:id=>1, :numb=>10}
+    @ds.filter(:numb=>:$n).call([:map, :numb], :n=>10).should == [10]
+    @ds.filter(:numb=>:$n).call([:to_hash, :id, :numb], :n=>10).should == {1=>10}
+    @ds.filter(:numb=>:$n).call([:to_hash_groups, :id, :numb], :n=>10).should == {1=>[10]}
   end
     
-  specify "should support blocks for select and all" do
+  specify "should support blocks for each, select, all, and map when using bound variables" do
+    a = []
+    @ds.filter(:numb=>:$n).call(:each, :n=>10){|r| r[:numb] *= 2; a << r}; a.should == [{:id=>1, :numb=>20}]
     @ds.filter(:numb=>:$n).call(:select, :n=>10){|r| r[:numb] *= 2}.should == [{:id=>1, :numb=>20}]
     @ds.filter(:numb=>:$n).call(:all, :n=>10){|r| r[:numb] *= 2}.should == [{:id=>1, :numb=>20}]
+    @ds.filter(:numb=>:$n).call([:map], :n=>10){|r| r[:numb] * 2}.should == [20]
   end
     
   specify "should support binding variables before the call with #bind" do
@@ -66,6 +73,10 @@ describe "Prepared Statements and Bound Arguments" do
     @ds.filter(:id=>:$i).filter(:numb=>@ds.select(:numb).filter(:numb=>:$n)).filter(:id=>:$j).call(:select, :n=>10, :i=>1, :j=>1).should == [{:id=>1, :numb=>10}]
   end
 
+  specify "should support subselects with exists with call" do
+    @ds.filter(:id=>:$i).filter(@ds.select(:numb).filter(:numb=>:$n).exists).filter(:id=>:$j).call(:select, :n=>10, :i=>1, :j=>1).should == [{:id=>1, :numb=>10}]
+  end
+
   specify "should support subselects with literal strings with call" do
     @ds.filter(:id=>:$i, :numb=>@ds.select(:numb).filter("numb = ?", :$n)).call(:select, :n=>10, :i=>1).should == [{:id=>1, :numb=>10}]
   end
@@ -112,20 +123,39 @@ describe "Prepared Statements and Bound Arguments" do
   end
 
   specify "should support bound variables with update" do
-    @ds.filter(:numb=>:$n).call(:update, {:n=>10, :nn=>20}, :numb=>:numb+:$nn).should == 1
+    @ds.filter(:numb=>:$n).call(:update, {:n=>10, :nn=>20}, :numb=>Sequel.+(:numb, :$nn)).should == 1
     @ds.all.should == [{:id=>1, :numb=>30}]
   end
   
-  specify "should support prepared statements with select, first, and all" do
+  specify "should support prepared statements when selecting" do
+    @ds.filter(:numb=>:$n).prepare(:each, :select_n)
+    @db.call(:select_n, :n=>10){|h| h.should == {:id=>1, :numb=>10}}
     @ds.filter(:numb=>:$n).prepare(:select, :select_n)
     @db.call(:select_n, :n=>10).should == [{:id=>1, :numb=>10}]
     @ds.filter(:numb=>:$n).prepare(:all, :select_n)
     @db.call(:select_n, :n=>10).should == [{:id=>1, :numb=>10}]
     @ds.filter(:numb=>:$n).prepare(:first, :select_n)
     @db.call(:select_n, :n=>10).should == {:id=>1, :numb=>10}
+    @ds.filter(:numb=>:$n).prepare([:map, :numb], :select_n)
+    @db.call(:select_n, :n=>10).should == [10]
+    @ds.filter(:numb=>:$n).prepare([:to_hash, :id, :numb], :select_n)
+    @db.call(:select_n, :n=>10).should == {1=>10}
   end
 
-  specify "should support prepared statements being call multiple times with different arguments" do
+  specify "should support blocks for each, select, all, and map when using prepared statements" do
+    a = []
+    @ds.filter(:numb=>:$n).prepare(:each, :select_n).call(:n=>10){|r| r[:numb] *= 2; a << r}; a.should == [{:id=>1, :numb=>20}]
+    a = []
+    @db.call(:select_n, :n=>10){|r| r[:numb] *= 2; a << r}; a.should == [{:id=>1, :numb=>20}]
+    @ds.filter(:numb=>:$n).prepare(:select, :select_n).call(:n=>10){|r| r[:numb] *= 2}.should == [{:id=>1, :numb=>20}]
+    @db.call(:select_n, :n=>10){|r| r[:numb] *= 2}.should == [{:id=>1, :numb=>20}]
+    @ds.filter(:numb=>:$n).prepare(:all, :select_n).call(:n=>10){|r| r[:numb] *= 2}.should == [{:id=>1, :numb=>20}]
+    @db.call(:select_n, :n=>10){|r| r[:numb] *= 2}.should == [{:id=>1, :numb=>20}]
+    @ds.filter(:numb=>:$n).prepare([:map], :select_n).call(:n=>10){|r| r[:numb] *= 2}.should == [20]
+    @db.call(:select_n, :n=>10){|r| r[:numb] *= 2}.should == [20]
+  end
+    
+  specify "should support prepared statements being called multiple times with different arguments" do
     @ds.filter(:numb=>:$n).prepare(:select, :select_n)
     @db.call(:select_n, :n=>10).should == [{:id=>1, :numb=>10}]
     @db.call(:select_n, :n=>0).should == []
@@ -151,6 +181,10 @@ describe "Prepared Statements and Bound Arguments" do
     @ds.filter(:id=>:$i).filter(:numb=>@ds.select(:numb).filter(:numb=>:$n)).filter(:id=>:$j).prepare(:select, :seq_select).call(:n=>10, :i=>1, :j=>1).should == [{:id=>1, :numb=>10}]
   end
 
+  specify "should support subselects with exists with prepare" do
+    @ds.filter(:id=>:$i).filter(@ds.select(:numb).filter(:numb=>:$n).exists).filter(:id=>:$j).prepare(:select, :seq_select).call(:n=>10, :i=>1, :j=>1).should == [{:id=>1, :numb=>10}]
+  end
+
   specify "should support subselects with literal strings with prepare" do
     @ds.filter(:id=>:$i, :numb=>@ds.select(:numb).filter("numb = ?", :$n)).prepare(:select, :seq_select).call(:n=>10, :i=>1).should == [{:id=>1, :numb=>10}]
   end
@@ -201,7 +235,7 @@ describe "Prepared Statements and Bound Arguments" do
   end
 
   specify "should support prepared statements with update" do
-    @ds.filter(:numb=>:$n).prepare(:update, :update_n, :numb=>:numb+:$nn)
+    @ds.filter(:numb=>:$n).prepare(:update, :update_n, :numb=>Sequel.+(:numb, :$nn))
     @db.call(:update_n, :n=>10, :nn=>20).should == 1
     @ds.all.should == [{:id=>1, :numb=>30}]
   end
@@ -223,7 +257,7 @@ describe "Prepared Statements and Bound Arguments" do
 end
 
 describe "Bound Argument Types" do
-  before do
+  before(:all) do
     @db = INTEGRATION_DB
     @db.create_table!(:items) do
       primary_key :id
@@ -237,31 +271,61 @@ describe "Bound Argument Types" do
     end
     @ds = @db[:items]
     @vs = {:d=>Date.civil(2010, 10, 11), :dt=>DateTime.civil(2010, 10, 12, 13, 14, 15), :f=>1.0, :s=>'str', :t=>Time.at(20101010), :file=>Sequel::SQL::Blob.new('blob'), :b=>true}
+  end
+  before do
+    @ds.delete
     @ds.insert(@vs)
-    @ds.meta_def(:ba) do |sym, type|
-      prepared_arg_placeholder == '$' ? :"#{sym}__#{type}" : sym
-    end
   end
   after do
     Sequel.datetime_class = Time
-    @db.drop_table(:items)
+  end
+  after(:all) do
+    @db.drop_table?(:items)
   end
 
   cspecify "should handle date type", [:do, :sqlite], :mssql, [:jdbc, :sqlite], :oracle do 
     @ds.filter(:d=>:$x).prepare(:first, :ps_date).call(:x=>@vs[:d])[:d].should == @vs[:d]
   end
 
-  cspecify "should handle datetime type", [:do], [:mysql2], [:swift], [:jdbc, :sqlite], [:tinytds], [:oracle] do
+  cspecify "should handle datetime type", [:do], [:mysql2], [:jdbc, :sqlite], [:tinytds], [:oracle] do
     Sequel.datetime_class = DateTime
     @ds.filter(:dt=>:$x).prepare(:first, :ps_datetime).call(:x=>@vs[:dt])[:dt].should == @vs[:dt]
   end
 
-  cspecify "should handle time type", [:do], [:jdbc, :sqlite] do
+  cspecify "should handle datetime type with fractional seconds", [:do], [:mysql2], [:jdbc, :sqlite], [:tinytds], [:oracle] do
+    Sequel.datetime_class = DateTime
+    fract_time = DateTime.parse('2010-10-12 13:14:15.500000')
+    @ds.prepare(:update, :ps_datetime_up, :dt=>:$x).call(:x=>fract_time)
+    @ds.literal(@ds.filter(:dt=>:$x).prepare(:first, :ps_datetime).call(:x=>fract_time)[:dt]).should == @ds.literal(fract_time)
+  end
+
+  cspecify "should handle time type", [:do], [:jdbc, :sqlite], [:swift], [:oracle] do
     @ds.filter(:t=>:$x).prepare(:first, :ps_time).call(:x=>@vs[:t])[:t].should == @vs[:t]
   end
 
-  cspecify "should handle blob type", [:swift], [:odbc], [:jdbc, :db2], :oracle, :derby do
-    @ds.filter(:file=>:$x).prepare(:first, :ps_blob).call(:x=>@vs[:file])[:file].should == @vs[:file]
+  cspecify "should handle time type with fractional seconds", [:do], [:jdbc, :sqlite], [:oracle], [:swift, :postgres] do
+    fract_time = @vs[:t] + 0.5
+    @ds.prepare(:update, :ps_time_up, :t=>:$x).call(:x=>fract_time)
+    @ds.literal(@ds.filter(:t=>:$x).prepare(:first, :ps_time).call(:x=>fract_time)[:t]).should == @ds.literal(fract_time)
+  end
+
+  cspecify "should handle blob type", [:odbc], [:oracle] do
+    @ds.delete
+    @ds.prepare(:insert, :ps_blob, {:file=>:$x}).call(:x=>@vs[:file])
+    @ds.get(:file).should == @vs[:file]
+  end
+
+  cspecify "should handle blob type with nil values", [:oracle], [:tinytds], [:jdbc, proc{|db| defined?(Sequel::JDBC::SQLServer::DatabaseMethods) && db.is_a?(Sequel::JDBC::SQLServer::DatabaseMethods)}] do
+    @ds.delete
+    @ds.prepare(:insert, :ps_blob, {:file=>:$x}).call(:x=>nil)
+    @ds.get(:file).should == nil
+  end
+
+  cspecify "should handle blob type with embedded zeros", [:odbc], [:oracle] do
+    zero_blob = Sequel::SQL::Blob.new("a\0"*100)
+    @ds.delete
+    @ds.prepare(:insert, :ps_blob, {:file=>:$x}).call(:x=>zero_blob)
+    @ds.get(:file).should == zero_blob
   end
 
   cspecify "should handle float type", [:swift, :sqlite] do
@@ -275,7 +339,7 @@ describe "Bound Argument Types" do
   cspecify "should handle boolean type", [:do, :sqlite], [:odbc, :mssql], [:jdbc, :sqlite], [:jdbc, :db2], :oracle do
     @ds.filter(:b=>:$x).prepare(:first, :ps_string).call(:x=>@vs[:b])[:b].should == @vs[:b]
   end
-end unless Sequel.guarded?([:swift, :postgres])
+end
 
 describe "Dataset#unbind" do
   before do
@@ -289,7 +353,7 @@ describe "Dataset#unbind" do
     @u = proc{|ds| ds, bv = ds.unbind; ds.call(:first, bv)}
   end
   after do
-    INTEGRATION_DB.drop_table(:items) rescue nil
+    INTEGRATION_DB.drop_table?(:items)
   end
   
   specify "should unbind values assigned to equality and inequality statements" do
@@ -317,7 +381,7 @@ describe "Dataset#unbind" do
     @u[@ds.filter{c > 0}].should == {:c=>BigDecimal.new('1.1')}
   end
 
-  cspecify "should handle dates and times", [:do], [:jdbc, :mssql], [:jdbc, :sqlite], [:tinytds], :oracle do
+  cspecify "should handle dates and times", [:do], [:jdbc, :mssql], [:jdbc, :sqlite], [:swift], [:tinytds], :oracle do
     @ct[Date, Date.today]
     @u[@ds.filter(:c=>Date.today)].should == {:c=>Date.today}
     t = Time.now
@@ -338,8 +402,8 @@ describe "Dataset#unbind" do
       Integer :d
     end
     @ds.insert(:a=>2, :b=>0, :c=>3, :d=>5)
-    @u[@ds.filter{a > 1}.and{b < 2}.or(:c=>3).and({~{:d=>4}=>1}.case(0) => 1)].should == {:a=>2, :b=>0, :c=>3, :d=>5}
-    @u[@ds.filter{a > 1}.and{b < 2}.or(:c=>3).and({~{:d=>5}=>1}.case(0) => 1)].should == nil
+    @u[@ds.filter{a > 1}.and{b < 2}.or(:c=>3).and(Sequel.case({~Sequel.expr(:d=>4)=>1}, 0) => 1)].should == {:a=>2, :b=>0, :c=>3, :d=>5}
+    @u[@ds.filter{a > 1}.and{b < 2}.or(:c=>3).and(Sequel.case({~Sequel.expr(:d=>5)=>1}, 0) => 1)].should == nil
   end
 
   specify "should handle case where the same variable has the same value in multiple places " do
