@@ -2,7 +2,7 @@ require 'rubygems'
 require 'logger'
 unless Object.const_defined?('Sequel')
   $:.unshift(File.join(File.dirname(File.expand_path(__FILE__)), "../../lib/"))
-  require 'sequel'
+  require 'sequel/no_core_ext'
 end
 begin
   require File.join(File.dirname(File.dirname(__FILE__)), 'spec_config.rb') unless defined?(INTEGRATION_DB)
@@ -86,7 +86,7 @@ else
   INTEGRATION_DB = Sequel.sqlite
 end
 
-if INTEGRATION_DB.adapter_scheme == :ibmdb
+if INTEGRATION_DB.adapter_scheme == :ibmdb || (INTEGRATION_DB.adapter_scheme == :ado && INTEGRATION_DB.database_type == :access)
   def INTEGRATION_DB.drop_table(*tables)
     super
   rescue Sequel::DatabaseError
@@ -94,3 +94,10 @@ if INTEGRATION_DB.adapter_scheme == :ibmdb
     super
   end
 end
+
+if ENV['SEQUEL_CONNECTION_VALIDATOR']
+  ENV['SEQUEL_NO_CHECK_SQLS'] = '1'
+  INTEGRATION_DB.extension(:connection_validator)
+  INTEGRATION_DB.pool.connection_validation_timeout = -1
+end
+
