@@ -1,55 +1,47 @@
 require File.join(File.dirname(File.expand_path(__FILE__)), "spec_helper")
 
 describe "Model#after_initialize" do
-  specify "should be called after initialization" do
-    $values1 = nil
-    $reached_after_initialized = false
+  specify "should be called after initialization for both new object and objects retrieved from the database" do
     
-    a = Class.new(Sequel::Model)
-    a.class_eval do
+    a = Class.new(Sequel::Model) do
+      self::Foo = []
       columns :x, :y
       def after_initialize
-        $values1 = @values.clone
-        $reached_after_initialized = true
+        model::Foo << @values.clone
       end
     end
     
     a.new(:x => 1, :y => 2)
-    $values1.should == {:x => 1, :y => 2}
-    $reached_after_initialized.should == true
+    a.call(:x => 1, :y => 3)
+    a::Foo.should == [{:x => 1, :y => 2}, {:x => 1, :y => 3}]
   end
 end
 
 describe "Model#before_create && Model#after_create" do
   before do
-    MODEL_DB.reset
-
-    @c = Class.new(Sequel::Model(:items))
-    @c.class_eval do
+    @c = Class.new(Sequel::Model(:items)) do
       columns :x
-      no_primary_key
+      set_primary_key :x
+      unrestrict_primary_key
       
       def after_create
         MODEL_DB << "BLAH after"
       end
     end
+    MODEL_DB.reset
   end
   
   specify "should be called around new record creation" do
     @c.send(:define_method, :before_create){MODEL_DB << "BLAH before"}
     @c.create(:x => 2)
-    MODEL_DB.sqls.should == [
-      'BLAH before',
-      'INSERT INTO items (x) VALUES (2)',
-      'BLAH after'
-    ]
+    MODEL_DB.sqls.should == ['BLAH before', 'INSERT INTO items (x) VALUES (2)', 'BLAH after', 'SELECT * FROM items WHERE (x = 2) LIMIT 1']
   end
 
   specify ".create should cancel the save and raise an error if before_create returns false and raise_on_save_failure is true" do
     @c.send(:define_method, :before_create){false}
-    proc{@c.load(:id => 2233).save}.should_not raise_error(Sequel::ValidationFailed)
     proc{@c.create(:x => 2)}.should raise_error(Sequel::BeforeHookFailed)
     MODEL_DB.sqls.should == []
+    proc{@c.load(:id => 2233).save}.should_not raise_error(Sequel::ValidationFailed)
   end
 
   specify ".create should cancel the save and return nil if before_create returns false and raise_on_save_failure is false" do
@@ -62,24 +54,20 @@ end
 
 describe "Model#before_update && Model#after_update" do
   before do
-    MODEL_DB.reset
-
-    @c = Class.new(Sequel::Model(:items))
-    @c.class_eval do
+    @c = Class.new(Sequel::Model(:items)) do
       columns :id, :x
-      def after_update; MODEL_DB << "BLAH after" end
+      def after_update
+        MODEL_DB << "BLAH after"
+      end
     end
+    MODEL_DB.reset
   end
   
   specify "should be called around record update" do
     @c.send(:define_method, :before_update){MODEL_DB << "BLAH before"}
     m = @c.load(:id => 2233, :x=>123)
     m.save
-    MODEL_DB.sqls.should == [
-      'BLAH before',
-      'UPDATE items SET x = 123 WHERE (id = 2233)',
-      'BLAH after'
-    ]
+    MODEL_DB.sqls.should == ['BLAH before', 'UPDATE items SET x = 123 WHERE (id = 2233)', 'BLAH after']
   end
 
   specify "#save should cancel the save and raise an error if before_update returns false and raise_on_save_failure is true" do
@@ -107,35 +95,28 @@ end
 
 describe "Model#before_save && Model#after_save" do
   before do
-    MODEL_DB.reset
-
-    @c = Class.new(Sequel::Model(:items))
-    @c.class_eval do
+    @c = Class.new(Sequel::Model(:items)) do
       columns :x
-      def after_save; MODEL_DB << "BLAH after" end
+      def after_save
+        MODEL_DB << "BLAH after"
+      end
     end
+    MODEL_DB.reset
   end
   
   specify "should be called around record update" do
     @c.send(:define_method, :before_save){MODEL_DB << "BLAH before"}
     m = @c.load(:id => 2233, :x=>123)
     m.save
-    MODEL_DB.sqls.should == [
-      'BLAH before',
-      'UPDATE items SET x = 123 WHERE (id = 2233)',
-      'BLAH after'
-    ]
+    MODEL_DB.sqls.should == ['BLAH before', 'UPDATE items SET x = 123 WHERE (id = 2233)', 'BLAH after']
   end
   
   specify "should be called around record creation" do
     @c.send(:define_method, :before_save){MODEL_DB << "BLAH before"}
-    @c.no_primary_key
+    @c.set_primary_key :x
+    @c.unrestrict_primary_key
     @c.create(:x => 2)
-    MODEL_DB.sqls.should == [
-      'BLAH before',
-      'INSERT INTO items (x) VALUES (2)',
-      'BLAH after'
-    ]
+    MODEL_DB.sqls.should == ['BLAH before', 'INSERT INTO items (x) VALUES (2)', 'BLAH after', 'SELECT * FROM items WHERE (x = 2) LIMIT 1']
   end
 
   specify "#save should cancel the save and raise an error if before_save returns false and raise_on_save_failure is true" do
@@ -159,31 +140,29 @@ describe "Model#before_save && Model#after_save" do
     @c.load(:id => 2233).save.should == nil
     MODEL_DB.sqls.should == []
   end
+
+  specify "#save should have a raised exception reference the model instance" do
+    @c.send(:define_method, :before_save){false}
+    proc{@c.create(:x => 2233)}.should raise_error(Sequel::HookFailed){|e| e.model.should == @c.load(:x=>2233)}
+    MODEL_DB.sqls.should == []
+  end
 end
 
 describe "Model#before_destroy && Model#after_destroy" do
   before do
-    MODEL_DB.reset
-
-    @c = Class.new(Sequel::Model(:items))
-    @c.class_eval do
-      def after_destroy; MODEL_DB << "BLAH after"; end
-      
-      def delete
-        MODEL_DB << "DELETE BLAH"
+    @c = Class.new(Sequel::Model(:items)) do
+      def after_destroy
+        MODEL_DB << "BLAH after"
       end
     end
+    MODEL_DB.reset
   end
   
   specify "should be called around record destruction" do
     @c.send(:define_method, :before_destroy){MODEL_DB << "BLAH before"}
     m = @c.load(:id => 2233)
     m.destroy
-    MODEL_DB.sqls.should == [
-      'BLAH before',
-      'DELETE BLAH',
-      'BLAH after'
-    ]
+    MODEL_DB.sqls.should == ['BLAH before', 'DELETE FROM items WHERE id = 2233', 'BLAH after']
   end
 
   specify "#destroy should cancel the destroy and raise an error if before_destroy returns false and raise_on_save_failure is true" do
@@ -209,16 +188,15 @@ end
 
 describe "Model#before_validation && Model#after_validation" do
   before do
-    MODEL_DB.reset
-
-    @c = Class.new(Sequel::Model(:items))
-    @c.class_eval do
-      def after_validation; MODEL_DB << "BLAH after" end
+    @c = Class.new(Sequel::Model(:items)) do
+      columns :id
+      def after_validation
+        MODEL_DB << "BLAH after"
+      end
 
       def validate
         errors.add(:id, 'not valid') unless id == 2233
       end
-      columns :id
     end
   end
   
@@ -228,7 +206,6 @@ describe "Model#before_validation && Model#after_validation" do
     m.should be_valid
     MODEL_DB.sqls.should == ['BLAH before', 'BLAH after']
 
-    MODEL_DB.sqls.clear
     m = @c.load(:id => 22)
     m.should_not be_valid
     MODEL_DB.sqls.should == ['BLAH before', 'BLAH after']
@@ -240,7 +217,6 @@ describe "Model#before_validation && Model#after_validation" do
     m.save.should == m
     MODEL_DB.sqls.should == ['BLAH before', 'BLAH after', 'UPDATE items SET x = 123 WHERE (id = 2233)']
 
-    MODEL_DB.sqls.clear
     m = @c.load(:id => 22)
     m.raise_on_save_failure = false
     m.save.should == nil
@@ -277,13 +253,10 @@ end
 
 describe "Model around filters" do
   before do
-    MODEL_DB.reset
-
-    @c = Class.new(Sequel::Model(:items))
-    @c.class_eval do
+    @c = Class.new(Sequel::Model(:items)) do
       columns :id, :x
-      def _save_refresh(*a) end
     end
+    MODEL_DB.reset
   end
   
   specify "around_create should be called around new record creation" do
@@ -295,7 +268,7 @@ describe "Model around filters" do
       end
     end
     @c.create(:x => 2)
-    MODEL_DB.sqls.should == [ 'ac_before', 'INSERT INTO items (x) VALUES (2)', 'ac_after' ]
+    MODEL_DB.sqls.should == ['ac_before', 'INSERT INTO items (x) VALUES (2)', 'ac_after', "SELECT * FROM items WHERE (id = 10) LIMIT 1"]
   end
 
   specify "around_delete should be called around record destruction" do
@@ -307,7 +280,7 @@ describe "Model around filters" do
       end
     end
     @c.load(:id=>1, :x => 2).destroy
-    MODEL_DB.sqls.should == [ 'ad_before', 'DELETE FROM items WHERE (id = 1)', 'ad_after' ]
+    MODEL_DB.sqls.should == ['ad_before', 'DELETE FROM items WHERE id = 1', 'ad_after']
   end
   
   specify "around_update should be called around updating existing records" do
@@ -319,10 +292,10 @@ describe "Model around filters" do
       end
     end
     @c.load(:id=>1, :x => 2).save
-    MODEL_DB.sqls.should == [ 'au_before', 'UPDATE items SET x = 2 WHERE (id = 1)', 'au_after' ]
+    MODEL_DB.sqls.should == ['au_before', 'UPDATE items SET x = 2 WHERE (id = 1)', 'au_after']
   end
 
-  specify "around_update should be called around saving both new and existing records, around either after_create and after_update" do
+  specify "around_save should be called around saving both new and existing records, around either after_create and after_update" do
     @c.class_eval do
       def around_update
         MODEL_DB << 'au_before'
@@ -341,10 +314,9 @@ describe "Model around filters" do
       end
     end
     @c.create(:x => 2)
-    MODEL_DB.sqls.should == [ 'as_before', 'ac_before', 'INSERT INTO items (x) VALUES (2)', 'ac_after', 'as_after' ]
-    MODEL_DB.sqls.clear
+    MODEL_DB.sqls.should == ['as_before', 'ac_before', 'INSERT INTO items (x) VALUES (2)', 'ac_after', 'as_after', "SELECT * FROM items WHERE (id = 10) LIMIT 1"]
     @c.load(:id=>1, :x => 2).save
-    MODEL_DB.sqls.should == [ 'as_before', 'au_before', 'UPDATE items SET x = 2 WHERE (id = 1)', 'au_after', 'as_after' ]
+    MODEL_DB.sqls.should == ['as_before', 'au_before', 'UPDATE items SET x = 2 WHERE (id = 1)', 'au_after', 'as_after']
   end
 
   specify "around_validation should be called around validating records" do
@@ -413,20 +385,175 @@ describe "Model around filters" do
     @c.raise_on_save_failure = false
 
     o = @c.load(:id => 1)
-    o.meta_def(:around_save){}
+    def o.around_save() end
     o.save.should == nil
 
     o = @c.load(:id => 1)
-    o.meta_def(:around_update){}
+    def o.around_update() end
     o.save.should == nil
 
     o = @c.new
-    o.meta_def(:around_create){}
+    def o.around_create() end
     o.save.should == nil
 
     o = @c.new
-    o.meta_def(:around_validation){}
+    def o.around_validation() end
     o.save.should == nil
   end
+end
 
+describe "Model#after_commit and #after_rollback" do
+  before do
+    @db = Sequel.mock(:servers=>{:test=>{}})
+    @m = Class.new(Sequel::Model(@db[:items])) do
+      attr_accessor :rb
+      def _delete
+      end
+      def after_save
+        db.execute('as')
+        raise Sequel::Rollback if rb
+      end
+      def after_commit
+        db.execute('ac')
+      end
+      def after_rollback
+        db.execute('ar')
+      end
+      def after_destroy
+        db.execute('ad')
+        raise Sequel::Rollback if rb
+      end
+      def after_destroy_commit
+        db.execute('adc')
+      end
+      def after_destroy_rollback
+        db.execute('adr')
+      end
+    end
+    @m.use_transactions = true
+    @o = @m.load({})
+    @db.sqls
+  end
+
+  specify "should call after_commit for save after the transaction commits if it commits" do
+    @o.save
+    @db.sqls.should == ['BEGIN', 'as', 'COMMIT', 'ac']
+  end
+
+  specify "should call after_rollback for save after the transaction rolls back if it rolls back" do
+    @o.rb = true
+    @o.save
+    @db.sqls.should == ['BEGIN', 'as', 'ROLLBACK', 'ar']
+  end
+
+  specify "should have after_commit respect any surrounding transactions" do
+    @db.transaction do
+      @o.save
+    end
+    @db.sqls.should == ['BEGIN', 'as', 'COMMIT', 'ac']
+  end
+
+  specify "should have after_rollback respect any surrounding transactions" do
+    @db.transaction do
+      @o.rb = true
+      @o.save
+    end
+    @db.sqls.should == ['BEGIN', 'as', 'ROLLBACK', 'ar']
+  end
+
+  specify "should have after_commit work with surrounding transactions and sharding" do
+    @db.transaction(:server=>:test) do
+      @o.save
+    end
+    @db.sqls.should == ['BEGIN -- test', 'BEGIN', 'as', 'COMMIT', 'ac', 'COMMIT -- test']
+  end
+
+  specify "should have after_rollback work with surrounding transactions and sharding" do
+    @db.transaction(:server=>:test) do
+      @o.rb = true
+      @o.save
+    end
+    @db.sqls.should == ['BEGIN -- test', 'BEGIN', 'as', 'ROLLBACK', 'ar', 'COMMIT -- test']
+  end
+
+  specify "should call after_destroy_commit for destroy after the transaction commits if it commits" do
+    @o.destroy
+    @db.sqls.should == ['BEGIN', 'ad', 'COMMIT', 'adc']
+  end
+
+  specify "should call after_destroy_rollback for destroy after the transaction rolls back if it rolls back" do
+    @o.rb = true
+    @o.destroy
+    @db.sqls.should == ['BEGIN', 'ad', 'ROLLBACK', 'adr']
+  end
+
+  specify "should have after_destroy_commit respect any surrounding transactions" do
+    @db.transaction do
+      @o.destroy
+    end
+    @db.sqls.should == ['BEGIN', 'ad', 'COMMIT', 'adc']
+  end
+
+  specify "should have after_destroy_rollback respect any surrounding transactions" do
+    @db.transaction do
+      @o.rb = true
+      @o.destroy
+    end
+    @db.sqls.should == ['BEGIN', 'ad', 'ROLLBACK', 'adr']
+  end
+
+  specify "should have after_destroy commit work with surrounding transactions and sharding" do
+    @db.transaction(:server=>:test) do
+      @o.destroy
+    end
+    @db.sqls.should == ['BEGIN -- test', 'BEGIN', 'ad', 'COMMIT', 'adc', 'COMMIT -- test']
+  end
+
+  specify "should have after_destroy_rollback work with surrounding transactions and sharding" do
+    @db.transaction(:server=>:test) do
+      @o.rb = true
+      @o.destroy
+    end
+    @db.sqls.should == ['BEGIN -- test', 'BEGIN', 'ad', 'ROLLBACK', 'adr', 'COMMIT -- test']
+  end
+
+  specify "should not call after_commit if use_after_commit_rollback is false" do
+    @o.use_after_commit_rollback = false
+    @o.save
+    @db.sqls.should == ['BEGIN', 'as', 'COMMIT']
+  end
+
+  specify "should not call after_rollback if use_after_commit_rollback is false" do
+    @o.use_after_commit_rollback = false
+    @o.rb = true
+    @o.save
+    @db.sqls.should == ['BEGIN', 'as', 'ROLLBACK']
+  end
+
+  specify "should not call after_destroy_commit if use_after_commit_rollback is false" do
+    @o.use_after_commit_rollback = false
+    @o.destroy
+    @db.sqls.should == ['BEGIN', 'ad', 'COMMIT']
+  end
+
+  specify "should not call after_destroy_rollback for save if use_after_commit_rollback is false" do
+    @o.use_after_commit_rollback = false
+    @o.rb = true
+    @o.destroy
+    @db.sqls.should == ['BEGIN', 'ad', 'ROLLBACK']
+  end
+
+  specify "should handle use_after_commit_rollback at the class level" do
+    @m.use_after_commit_rollback = false
+    @o.save
+    @db.sqls.should == ['BEGIN', 'as', 'COMMIT']
+  end
+
+  specify "should handle use_after_commit_rollback when subclassing" do
+    @m.use_after_commit_rollback = false
+    o = Class.new(@m).load({})
+    @db.sqls
+    o.save
+    @db.sqls.should == ['BEGIN', 'as', 'COMMIT']
+  end
 end
